@@ -88,9 +88,10 @@ static const char *mount_known_locations [] = {
 	NULL
 };
 
-static const char *umount_known_locations [] = {
-	"/sbin/umount", "/bin/umount",
-	"/usr/sbin/umount", "/usr/bin/umount",
+static const char *pmount_known_locations [] = {
+	"/usr/sbin/pmount", "/usr/bin/pmount",
+	"/sbin/mount", "/bin/mount",
+	"/usr/sbin/mount", "/usr/bin/mount",
 	NULL
 };
 
@@ -101,8 +102,17 @@ static const char *pumount_known_locations [] = {
 	NULL
 };
 
+static const char *umount_known_locations [] = {
+	"/sbin/umount", "/bin/umount",
+	"/usr/sbin/umount", "/usr/bin/umount",
+	NULL
+};
+
+
 #define MOUNT_COMMAND mount_known_locations
 #define MOUNT_SEPARATOR " "
+#define PMOUNT_COMMAND pmount_known_locations
+#define PMOUNT_SEPARATOR " "
 #define UMOUNT_COMMAND umount_known_locations
 #define UMOUNT_SEPARATOR " "
 #define PUMOUNT_COMMAND pumount_known_locations
@@ -156,7 +166,6 @@ typedef struct _MountThreadAuth {
 } MountThreadAuth;
 
 
-#if !defined(USE_GNOME_MOUNT)
 /* gnome-mount programs display their own dialogs so no use for these functions */
 static char *
 generate_mount_error_message (char *standard_error,
@@ -214,7 +223,6 @@ generate_unmount_error_message (char *standard_error,
 
 	return message;
 }
-#endif /* !defined(USE_GNOME_MOUNT) */
 
 static void
 force_probe (void)
@@ -609,15 +617,17 @@ mount_unmount_thread (void *arg)
 	if (info->should_mount || info->should_unmount) {
 		error = NULL;
  		if (spawn_mount (info, 
-#if defined(USE_GNOME_MOUNT)
+#if defined(USE_HAL)
 				  /* do pass our environment when using gnome-mount progams */
 				  ((strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-mount") == 0) ||
 				   (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-umount") == 0) ||
-				   (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-eject") == 0)) ? NULL : envp,
-#elif defined(USE_HAL) && defined(HAL_MOUNT) && defined(HAL_UMOUNT)
+				   (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-eject") == 0) ||
+# if defined(HAL_MOUNT) && defined(HAL_UMOUNT)
 				  /* do pass our environment when using hal mount progams */
-				  ((strcmp (info->argv[0], HAL_MOUNT) == 0) ||
-				   (strcmp (info->argv[0], HAL_UMOUNT) == 0)) ? NULL : envp,
+				   (strcmp (info->argv[0], HAL_MOUNT) == 0) ||
+				   (strcmp (info->argv[0], HAL_UMOUNT) == 0) ||
+# endif
+				   FALSE) ? NULL : envp,
 #else
 				   envp,
 #endif
@@ -626,34 +636,40 @@ mount_unmount_thread (void *arg)
 				   &error)) {
 			if (exit_status != 0) {
 				info->succeeded = FALSE;
-#if defined(USE_GNOME_MOUNT)
-				/* gnome-mount programs display their own dialogs */
-				info->error_message = g_strdup ("");
-				info->detailed_error_message = g_strdup ("");
-#else
-				if (strlen (standard_error) > 0) {
-					if (info->should_mount) {
-						info->error_message = generate_mount_error_message (standard_error,
-												    info->device_type);
-					} else {
-						info->error_message = generate_unmount_error_message (standard_error,
-												      info->device_type);
-					}
-					info->detailed_error_message = g_strdup (standard_error);
-				} else {
-					/* As of 2.12 we introduce a new contract between gnome-vfs clients
-					 * invoking mount/unmount and the gnome-vfs-daemon:
-					 *
-					 *       "don't display an error dialog if error_message and
-					 *        detailed_error_message are both empty strings".
-					 *
-					 * We want this as we may use mount/unmount/ejects programs that
-					 * shows it's own dialogs. 
-					 */
+
+				if ((strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-mount") == 0) ||
+				    (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-umount") == 0) ||
+				    (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-eject") == 0)) {
+					/* gnome-mount programs display their own dialogs */
 					info->error_message = g_strdup ("");
 					info->detailed_error_message = g_strdup ("");
+				} else {
+					if (strlen (standard_error) > 0) {
+						if (info->should_mount) {
+							info->error_message = generate_mount_error_message (
+								standard_error,
+								info->device_type);
+						} else {
+							info->error_message = generate_unmount_error_message (
+								standard_error,
+								info->device_type);
+						}
+						info->detailed_error_message = g_strdup (standard_error);
+					} else {
+						/* As of 2.12 we introduce a new contract between gnome-vfs clients
+						 * invoking mount/unmount and the gnome-vfs-daemon:
+						 *
+						 *       "don't display an error dialog if error_message and
+						 *        detailed_error_message are both empty strings".
+						 *
+						 * We want this as we may use mount/unmount/ejects programs that
+						 * shows it's own dialogs. 
+						 */
+						info->error_message = g_strdup ("");
+						info->detailed_error_message = g_strdup ("");
+					}
 				}
-#endif
+
 			}
 
 			g_free (standard_error);
@@ -671,7 +687,7 @@ mount_unmount_thread (void *arg)
 
 		argv[0] = NULL;
 
-#if defined(USE_GNOME_MOUNT)
+#if defined(USE_HAL)
 		if (info->hal_udi != NULL) {
 			argv[0] = GNOME_VFS_BINDIR "/gnome-eject";
 			argv[1] = "--hal-udi";
@@ -681,8 +697,8 @@ mount_unmount_thread (void *arg)
 			if (!g_file_test (argv [0], G_FILE_TEST_IS_EXECUTABLE))
 				argv[0] = NULL;
 		}
-#elif defined(USE_HAL) && defined(HAL_EJECT)
-		if (info->hal_udi != NULL) {
+# if defined(HAL_EJECT)
+		if (argv[0] == NULL && info->hal_udi != NULL) {
 			argv[0] = HAL_EJECT;
 			argv[1] = info->device_path;
 			argv[2] = NULL;
@@ -690,6 +706,7 @@ mount_unmount_thread (void *arg)
 			if (!g_file_test (argv [0], G_FILE_TEST_IS_EXECUTABLE))
 				argv[0] = NULL;
 		}
+# endif
 #endif
 
 		if (argv[0] == NULL) {
@@ -725,14 +742,16 @@ mount_unmount_thread (void *arg)
 
 			if (exit_status != 0) {
 				info->succeeded = FALSE;
-#if defined(USE_GNOME_MOUNT)
-				/* gnome-mount programs display their own dialogs */
-				info->error_message = g_strdup ("");
-				info->detailed_error_message = g_strdup ("");
-#else
-				info->error_message = g_strdup (_("Unable to eject media"));
-				info->detailed_error_message = g_strdup (standard_error);
-#endif
+				if ((strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-mount") == 0) ||
+				    (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-umount") == 0) ||
+				    (strcmp (info->argv[0], GNOME_VFS_BINDIR "/gnome-eject") == 0)) {
+					/* gnome-mount programs display their own dialogs */
+					info->error_message = g_strdup ("");
+					info->detailed_error_message = g_strdup ("");
+				} else {
+					info->error_message = g_strdup (_("Unable to eject media"));
+					info->detailed_error_message = g_strdup (standard_error);
+				}
 			} else {
 				/* If the eject succeed then ignore the previous unmount error (if any) */
 				info->succeeded = TRUE;
@@ -777,8 +796,8 @@ mount_unmount_operation (const char *mount_point,
 	pthread_t mount_thread;
 	const char *name;
 	int i;
+	gboolean is_in_media;
 
-	
 #ifdef USE_VOLRMMOUNT
 	if (mount_point != NULL) {
 		name = strrchr (mount_point, '/');
@@ -799,47 +818,54 @@ mount_unmount_operation (const char *mount_point,
 # endif
 
 #endif
-       
-       if (should_mount) {
-#if defined(USE_GNOME_MOUNT)
+
+	is_in_media = mount_point != NULL ? g_str_has_prefix (mount_point, "/media") : FALSE;
+
+	command = NULL;
+
+	if (should_mount) {
+#if defined(USE_HAL)
 	       if (hal_udi != NULL && g_file_test (GNOME_VFS_BINDIR "/gnome-mount", G_FILE_TEST_IS_EXECUTABLE)) {
 		       command = GNOME_VFS_BINDIR "/gnome-mount";
 		       argument = "--hal-udi";
 		       name = hal_udi;
 	       } else {
-		       command = find_command (MOUNT_COMMAND);
+# if defined(HAL_MOUNT)
+		       if (hal_udi != NULL && g_file_test (HAL_MOUNT, G_FILE_TEST_IS_EXECUTABLE))
+			       command = HAL_MOUNT;
+		       else
+			       command = find_command (MOUNT_COMMAND);
+# else
+		       ;
+# endif
 	       }
-#elif defined(USE_HAL) && defined(HAL_MOUNT)
-	       if (hal_udi != NULL && g_file_test (HAL_MOUNT, G_FILE_TEST_IS_EXECUTABLE))
-		       command = HAL_MOUNT;
-	       else
-		       command = find_command (MOUNT_COMMAND);
-#else
-	       command = find_command (MOUNT_COMMAND);
 #endif
+	       if (command == NULL)
+		       command = find_command (is_in_media ? PMOUNT_COMMAND : MOUNT_COMMAND);
 #ifdef  MOUNT_ARGUMENT
 	       argument = MOUNT_ARGUMENT;
 #endif
        }
 
        if (should_unmount) {
-	       gboolean is_in_media = mount_point != NULL ? g_str_has_prefix (mount_point, "/media") : FALSE;
-#if defined(USE_GNOME_MOUNT)
+#if defined(USE_HAL)
 	       if (hal_udi != NULL && g_file_test (GNOME_VFS_BINDIR "/gnome-umount", G_FILE_TEST_IS_EXECUTABLE)) {
 		       command = GNOME_VFS_BINDIR "/gnome-umount";
 		       argument = "--hal-udi";
 		       name = hal_udi;
 	       } else {
-		       command = find_command (is_in_media ? PUMOUNT_COMMAND : UMOUNT_COMMAND);
-	       }
-#elif defined(USE_HAL) && defined(HAL_UMOUNT)
-	       if (hal_udi != NULL && g_file_test (HAL_UMOUNT, G_FILE_TEST_IS_EXECUTABLE))
-		       command = HAL_UMOUNT;
-	       else
-		       command = find_command (is_in_media ? PUMOUNT_COMMAND : UMOUNT_COMMAND);
-#else
-	       command = find_command (is_in_media ? PUMOUNT_COMMAND : UMOUNT_COMMAND);
+# if defined(HAL_UMOUNT)
+		       if (hal_udi != NULL && g_file_test (HAL_UMOUNT, G_FILE_TEST_IS_EXECUTABLE))
+			       command = HAL_UMOUNT;
+		       else
+			       command = find_command (UMOUNT_COMMAND);
+# else
+		       ;
 #endif
+	       }
+#endif
+	       if (command == NULL)
+		       command = find_command (is_in_media ? PUMOUNT_COMMAND : UMOUNT_COMMAND);
 #ifdef  UNMOUNT_ARGUMENT
 		argument = UNMOUNT_ARGUMENT;
 #endif
@@ -1012,18 +1038,35 @@ gnome_vfs_volume_unmount (GnomeVFSVolume *volume,
 
 	type = gnome_vfs_volume_get_volume_type (volume);
 	if (type == GNOME_VFS_VOLUME_TYPE_MOUNTPOINT) {
+		char *hal_udi;
+
 		uri = gnome_vfs_volume_get_activation_uri (volume);
 		mount_path = gnome_vfs_get_local_path_from_uri (uri);
 		g_free (uri);
 		device_path = gnome_vfs_volume_get_device_path (volume);
+		hal_udi = gnome_vfs_volume_get_hal_udi (volume);
+
+		/* Volumes from drives that are not polled may not
+		 * have a hal_udi.. take the one from HAL to get
+		 * gnome-mount working */
+		if (hal_udi == NULL) {
+			GnomeVFSDrive *drive;
+			drive = gnome_vfs_volume_get_drive (volume);
+			if (drive != NULL) {
+				hal_udi = gnome_vfs_drive_get_hal_udi (drive);
+				gnome_vfs_drive_unref (drive);
+			}
+		}
+
 		mount_unmount_operation (mount_path,
 					 device_path,
-					 gnome_vfs_volume_get_hal_udi (volume),
+					 hal_udi,
 					 gnome_vfs_volume_get_device_type (volume),
 					 FALSE, TRUE, FALSE,
 					 callback, user_data);
 		g_free (mount_path);
 		g_free (device_path);
+		g_free (hal_udi);
 	} else if (type == GNOME_VFS_VOLUME_TYPE_VFS_MOUNT) {
 		/* left intentionally blank as these cannot be mounted and thus not unmounted */
 	} else if (type == GNOME_VFS_VOLUME_TYPE_CONNECTED_SERVER) {
@@ -1059,14 +1102,29 @@ gnome_vfs_volume_eject (GnomeVFSVolume *volume,
 		mount_path = gnome_vfs_get_local_path_from_uri (uri);
 		g_free (uri);
 		device_path = gnome_vfs_volume_get_device_path (volume);
+		hal_udi = gnome_vfs_volume_get_hal_udi (volume);
+
+		/* Volumes from drives that are not polled may not
+		 * have a hal_udi.. take the one from HAL to get
+		 * gnome-mount working */
+		if (hal_udi == NULL) {
+			GnomeVFSDrive *drive;
+			drive = gnome_vfs_volume_get_drive (volume);
+			if (drive != NULL) {
+				hal_udi = gnome_vfs_drive_get_hal_udi (drive);
+				gnome_vfs_drive_unref (drive);
+			}
+		}
+
 		mount_unmount_operation (mount_path,
 					 device_path,
-					 gnome_vfs_volume_get_hal_udi (volume),
+					 hal_udi,
 					 gnome_vfs_volume_get_device_type (volume),
 					 FALSE, TRUE, TRUE,
 					 callback, user_data);
 		g_free (mount_path);
 		g_free (device_path);
+		g_free (hal_udi);
 	} else if (type == GNOME_VFS_VOLUME_TYPE_VFS_MOUNT) {
 		hal_udi = gnome_vfs_volume_get_hal_udi (volume);
 		uri = gnome_vfs_volume_get_activation_uri (volume);
@@ -1085,6 +1143,7 @@ gnome_vfs_volume_eject (GnomeVFSVolume *volume,
 			g_free (device_path);
 		    }
 		g_free (uri);
+		g_free (hal_udi);
 	} else if (type == GNOME_VFS_VOLUME_TYPE_CONNECTED_SERVER) {
 		unmount_connected_server (volume, callback, user_data);
 	}
